package cn.com.qmth.archive.service.impl;

import java.io.File;
import java.io.IOException;
import java.time.Instant;
import java.time.LocalDateTime;
import java.time.ZoneId;
import java.util.ArrayList;
import java.util.Date;
import java.util.List;
import java.util.Map;

import org.apache.commons.collections4.CollectionUtils;
import org.apache.commons.io.FileUtils;
import org.apache.commons.lang3.StringUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import com.qmth.boot.core.exception.StatusException;

import cn.com.qmth.archive.bean.TableCols;
import cn.com.qmth.archive.config.SysProperty;
import cn.com.qmth.archive.dao.MarkingDao;
import cn.com.qmth.archive.service.MarkingService;
import cn.com.qmth.archive.utils.DateUtil;
import cn.com.qmth.archive.utils.FileUtil;

@Service
public class MarkingServiceImpl implements MarkingService {

    private final static Logger log = LoggerFactory.getLogger(MarkingService.class);

    @Autowired
    private SysProperty sysProperty;

    @Autowired
    private MarkingDao markingDao;

    @Override
    public void exportSql() {
        File dir = new File("./sql");
        if (dir.exists()) {
            FileUtil.deleteDirectory(dir);
        }
        dir.mkdirs();
        String workidStr = sysProperty.getWorkids();
        if (StringUtils.isBlank(workidStr)) {
            throw new StatusException("配置文件中workids为空");
        }
        List<Long> workids = new ArrayList<>();
        for (String wid : workidStr.split(",")) {
            if (StringUtils.isBlank(wid)) {
                throw new StatusException("配置文件中workids错误");
            }
            try {
                workids.add(Long.valueOf(wid));
            } catch (NumberFormatException e) {
                throw new StatusException("配置文件中workids错误");
            }
        }
        log.warn("*********************开始处理，workid：" + StringUtils.join(workids, ","));
        try {
            getWork(dir, workids);
            log.warn("*********************处理结束，成功");
        } catch (Exception e) {
            throw new StatusException("*********************处理结束，失败", e);
        }
    }

    private void getWork(File dir, List<Long> workids) throws IOException {
        log.warn("*********************处理work");
        List<Map<String, Object>> work = markingDao.getWork(TableCols.work, workids);
        if (CollectionUtils.isEmpty(work)) {
            throw new StatusException("work表没有数据");
        }
        File sql = new File(dir.getAbsolutePath() + "/work.sql");
        sql.createNewFile();
        List<String> ret = new ArrayList<>();
        for (Map<String, Object> map : work) {
            ret.add(getLine("ec_m_mark_work", TableCols.work, map));
        }
        FileUtils.writeLines(sql, "utf-8", ret, false);
    }

    private String getLine(String tbName, String[] cols, Map<String, Object> line) {
        StringBuilder sb = new StringBuilder("INSERT INTO `" + tbName + "` (`");
        sb.append(StringUtils.join(cols, "`,`") + "`) ");
        sb.append("values (");
        for (String col : cols) {
            Object ob = line.get(col);
            if (ob == null) {
                sb.append("null,");
            } else {
                if (ob instanceof Boolean) {
                    Boolean tem = (Boolean) ob;
                    if (tem) {
                        sb.append("1,");
                    } else {
                        sb.append("0,");
                    }
                } else if (ob instanceof LocalDateTime) {
                    LocalDateTime tem = (LocalDateTime) ob;
                    Instant instant = tem.atZone(ZoneId.systemDefault()).toInstant();

                    // 将 Instant 转换为 Date
                    Date date = Date.from(instant);
                    sb.append("'" + DateUtil.format(date, DateUtil.DatePatterns.CHINA_DEFAULT) + "',");
                } else if (ob instanceof String) {
                    sb.append("'" + ob.toString() + "',");
                } else {
                    sb.append(ob.toString() + ",");
                }
            }
        }
        sb.deleteCharAt(sb.length() - 1);
        sb.append("); ");
        return sb.toString();
    }

}